import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import * as React from "react";
import PropTypes from "prop-types";
import cx from "classnames";
import { useContext, } from "react";
import { useMeasure } from "./Hooks/use-measure";
import { PanelEnlargeCtx, useMMenuBarOcup } from "./MainMenuBar";
//缺省breaks ： <LineColumn column={2} breaks={[288, 520]}
//可以安排几个列 :要每列相等宽度，每一行高度也一相等的，最大支持8列（8块一排的）有意义的用途：编辑器某项目？。不是表格某一列的用途
//考虑到滚动条宽度或有或消失的扰动, 窗口大小可能遭遇的布局和测量扯皮死循环,通常手机屏幕宽度避免掉入;
const defaultBreakOf = [
    [360],
    [330, 1000],
    [310, 800, 1300],
    [280, 580, 1000, 1400],
    [250, 500, 800, 1150, 1500],
    [220, 420, 680, 980, 1300, 1600],
    [200, 380, 670, 920, 1150, 1450, 1750],
    [170, 340, 590, 820, 1100, 1300, 1580, 1880], //默认的配置最多支持八个，但也能允许自己扩展更多个的。
];
/**【3种思路】如何避免依据屏幕宽度做动态布局的效果，在遇到打印切换时刻，打印预览太草率地限定了纸张页数所导致的问题。
 * 1：每个LineColumn出现地方都应该配置打印A4。A4横板的。A3?几个常见打印目的地纸张的正常应该的效果：也就是suitableCls大小{a4:,a4L:,a3,}。
 * 2：直接约束准备打印浏览器的窗口宽度做法。
 * 3：完整的用css来调整的？【确保打印的正确性需要】只用css，不用JS做动态内容？测量div宽度做自适应的危险；
 * 重新加载？打印敲定页数之后才会再一次render的。根本没机会计算ref.div最新宽度甚至用beforePrintHandler来setSuitableCls也没有机会提前调整。
 * printComp[]；
 * 手机上chrome打印可能有些问题；
 * 若使用固定大小列style={{columnWidth: '270px', columnGap: '20px'}}布局，大小屏幕宽度缩放适应性不好，水平方向没法对齐，感觉凌乱！。
 * 若采用style={{display: 'grid', gridTemplateColumns: 'auto auto auto'}}，也有问题：列数量不能固定啊。
 * 1：2 1：1 框架调整，右边还是左边的半边框，默认编辑区底下空间100%，屏幕媒体宽度。应该部署有几个列的？？
 * */
export const LineColumn = ({ column: defaultColumn = 4, children, breaks = defaultBreakOf[defaultColumn - 1], inLeft, percent, ...other }) => {
    const contextPanel = useContext(PanelEnlargeCtx);
    const { carousel } = useMMenuBarOcup(false);
    //（全部列的）Viewpoint屏占比 系数
    let vppercent;
    if (percent !== undefined)
        vppercent = percent; //不启用useContext(PanelEnlargeCtx)+useMMenuBarOcup+inLeft的设置，采信外部直接控制的。
    else {
        const { enlarge } = contextPanel || {};
        //enlarge两半边的宽度协调:左边没有=0，1：左边右边1比2， 2： 1比1， 3： 2比1， 4：右边没有。
        vppercent = inLeft ? (enlarge === 1 ? 0.33 : enlarge === 3 ? 0.66 : enlarge === undefined ? 1 : 0.5) :
            (enlarge === 1 ? 0.66 : enlarge === 3 ? 0.33 : enlarge === undefined ? 1 : 0.5);
        if (contextPanel === null || carousel)
            vppercent = 1; //没有拆分为两个半边页面的框架形式，独立全撑开的页面。
        // console.log("打enlarge触发vppercent=",vppercent,enlarge,carousel);
    }
    //最大屏幕 可放下 安排 最多几个列数的； 非线性地比例降低列数?
    const column = breaks.length;
    if (column < 1)
        throw new Error("number of columns must >=1");
    /**反过来预设赋值 倒推合理的断开点;
     * 依据vppercent断定合适的CSS,
     * */
    const mediaQueries = React.useCallback((vppercent) => {
        if (vppercent <= 0)
            return {
                visibility: 'hidden'
            };
        const breakv = breaks.map(px => Math.round(px / vppercent));
        let csobj = {};
        for (let p = 0; p < column - 1; p++) {
            let key = '@media (min-width: ' + breakv[p] + 'px) and (max-width: ' + (breakv[p + 1] - 1) + 'px)';
            let col = p === 0 ? 1 : p + 1;
            csobj[key] = {
                gridTemplateColumns: 'repeat(' + col + ', auto)',
            };
        }
        let key = '@media (min-width: ' + breakv[column - 1] + 'px)';
        csobj[key] = {
            gridTemplateColumns: 'repeat(' + column + ', auto)',
        };
        // console.log("mediaQueries会了vppercent=",vppercent,breaks, csobj);
        return csobj;
    }, [column, breaks]);
    return (_jsx("div", { css: {
            display: 'grid',
            gridTemplateColumns: 'auto',
            //【自适应】grid-template-columns: repeat(auto-fit, minmax(min(200px, 100%), 1fr));  列宽度固定的。宽屏幕窄屏幕没有区分对待。
            ...mediaQueries(vppercent),
        }, ...other, children: children }));
};
/**动态布局通用的: 测量某一个参考DIV的宽度尺寸; 不走CSS做动态布局用的;
 * 假设使用useReferenceWidth配置 ref div的代码点和实际关联布局组件的代码点差不多是相接近层次的, 避免长距离组件嵌套的多层次地传递参数。?太多层次的用Context方式。
 * [组别注意]返回ref绑定的div约束: 不应该出现在<div css={{display: 'none',"@media print": { display: 'unset', },}}>这样子的上下文当中的;条件render小心;
 * */
export function useReferenceWidth() {
    const ref = React.useRef(null);
    //正常应该用的
    const rect = useMeasure(ref);
    //div被隐藏的上下文内部的::
    //有些情况需要用这个的？？
    // const rect= useMeasure(ref?.current? ref : null);       //没有加载reflc?.current的导致无法注册useResizeObserver，必须做一个三元。
    const width = rect?.width || 0;
    //返回的width时刻反馈变动;
    //这个返回的ref可以给LineColumnFlex做绑定,也能单独给任意附近宽度大小相似度高的其它div或空的div做绑定;
    return [ref, width];
}
;
/**需要用useReferenceWidth()配套的; flexBox做基础布局, 测量容器div宽度来自适应的。
 动态布局大问题！ 【特别注意】！
 只用css做动态的组件和用了js脚本做动态布局的组件在打印时刻不同，比如LineColumnFlex和LineColumn的差别即使这样，
 前者在打印时刻需要约束浏览器窗口的宽度为纸张输出的宽度差不多的大小，后者可打印页数就可能和内容页数不一致，后者只用css的就不会有这个麻烦。
 就算同一个组件有可能出现：同一次打印出现两种布局状态。比如这样的：<LineColumnFlex 1> {useMediaQuery({query: 'print'}) && <LineColumnFlex 2>}的，
 前面一个是js按照浏览器打印触发之前的div宽度来布局的有5列，然而后面哪一个是按照最终纸张输出宽度来做布局的才2列，完全不一样了！但是后面第二个组件可能输出不完整的，
 因为打印纸张的输出页数实际在useMediaQuery({query: 'print'})逻辑条件成立以前就敲定了可打印纸张数量！！所以有很大问题。js动态代码简单，css动态写代码麻烦。
 就接着这个例子讲：用{useMediaQuery({query: 'print'})&&逻辑嵌套的哪些内容，就算我约束了浏览器宽度为纸张宽度以后，实际上在打印敲定输出页数时间这些逻辑嵌套的
 需要打印内容实际上也并没有计算在立刻敲定输出页数之内的，因此也必然导致打印页数丢失或不够的问题的，这个情况必须用css,除非不打印比打印语境内容占用高度还少的话。
 <div css={{"@media not print": {display: 'none'}}}>
 <LineColumnFlex  column={7} />
 </div>
 如上代码会有严重毛病：打印输出实际上只有一次机会render；useMeasure(ref还是空的)div宽度=0导致实际只有输出一列的布局，打印没机会修正了，屏幕情况可以不限次数render，
 可是打印就不行了。这种情形若改成<div css={{visibility: 'hidden', "@media print": { visibility: 'unset' } }}>是能够确保打印能正常的输出，可惜的是屏幕，
 可屏幕遇见visibility=hidden还会保留位置空间的，显然是有问题的：空间占用？太乱了。
 [碰巧遭遇死循环]断点1150门槛：内容布局4列的后能反而省掉滚动台条1165px，被改成了5列反而要加滚动条啊div宽度1148px所以跌破门槛了再次调整布局，div测量被滚动条给扯了：扯皮了。
 加上了 scrollbarGutter: 'stable', 解决布局副作用的循环地跳变问题！可能死循环。但是固定的会手机触摸屏的固定预留滚动条的位置。
 【关键毛病】打印预览敲定页数之前没有经过新render的，直接根据屏幕当前的内容来确定的页数{display:none的div也可能计算在内的}。所以：这个问题像个不定时炸弹了。
 要被作为参考布局的div屏幕render语境中是绝对不能做{display:none}隐藏的,导致打印之前无法测量该div真实宽度,进一步导致布局问题,最后页数就不一致.把跟踪参考div独立到外部去;拆解功能依赖;
 这里参数 ref 可以是useReferenceWidth()直接绑定的,或者放空也行;
 拉拽窗口大小可能遭遇的布局和测量扯皮死循环现象: 在css设置上 {scrollbarGutter: 'stable'} 来避免滚动条的出现和消失给宽度测量布局场景的负反馈死循环.
 * */
export const LineColumnFlex = React.forwardRef(({ refWidth, column: defaultColumn = 4, children, breaks = defaultBreakOf[defaultColumn - 1], ...other }, ref) => {
    //最大屏幕 可放下 安排 最多几个列数的； 非线性地比例降低列数?
    const column = breaks.length;
    if (column < 1)
        throw new Error("number of columns must >=1");
    const suitableCls = React.useMemo(() => {
        let i = 0;
        for (; i < column; i++)
            if (refWidth < breaks[i])
                break;
        return i;
    }, [refWidth, column, breaks]);
    //加{scrollbarGutter: 'stable'},手机宽度狭窄会??  思路:智能记住和主动地消除?
    const itemCount = React.Children.count(children);
    const lineSum = Math.ceil(itemCount / suitableCls);
    //【问题来源】即使是display:none div包裹的情况suitableCls<=0，也应该争取设置即将要布局显示的内容，这些内容直接【决定了】打印预览的页数多少的，点击打印之后再做修改已经太迟了。
    const items = React.Children.map(children, (item, index) => {
        return React.cloneElement(item, {
            className: cx(['LineColumn-item',
                index === itemCount - 1 ? 'Litem-last' : ''
            ]),
            style: { flex: `1 1 ${Math.round(10000 / suitableCls) / 100}%` },
        });
    });
    const fakeSons = lineSum * suitableCls - itemCount;
    //必须加一层div{{ position: 'relative' }}，后面给出的父辈Div宽度width才对的。
    return (_jsxs("div", { ref: ref, style: { display: 'flex', flexWrap: 'wrap', position: 'relative', justifyContent: 'center', width: '-webkit-fill-available' }, ...other, children: [items, suitableCls > 0 && fakeSons > 0 && (new Array(fakeSons)).fill(null).map((each, clm) => {
                return _jsx("div", { css: { flex: `1 1 ${Math.round(10000 / suitableCls) / 100}%` } }, clm);
            })] }));
});
/**矛盾重重的版本, 没法完美解决的旧版本;  reflc没有抽离出去的;不用useReferenceWidth()配套;
 * */
const LineColumnFlex_OLD = ({ refWidth, column: defaultColumn = 4, children, breaks = defaultBreakOf[defaultColumn - 1], ...other }) => {
    const reflc = React.useRef(null);
    const column = breaks.length;
    if (column < 1)
        throw new Error("number of columns must >=1");
    if (refWidth <= 0)
        throw new Error("参考div宽度必须>0");
    const suitableCls = React.useMemo(() => {
        let i = 0;
        for (; i < column; i++)
            if (refWidth < breaks[i])
                break;
        return i;
    }, [refWidth, column, breaks]);
    const itemCount = React.Children.count(children);
    const lineSum = Math.ceil(itemCount / suitableCls);
    const items = React.Children.map(children, (item, index) => {
        return React.cloneElement(item, {
            className: cx(['LineColumn-item',
                index === itemCount - 1 ? 'Litem-last' : ''
            ]),
            //测试 style: suitableCls<=0? {flex: `1 1 33.33%` } : {flex: `1 1 ${Math.round(10000/suitableCls)/100}%` },
            style: { flex: `1 1 ${Math.round(10000 / suitableCls) / 100}%` },
        });
    });
    const fakeSons = lineSum * suitableCls - itemCount;
    return (_jsxs("div", { ref: reflc, style: { display: 'flex', flexWrap: 'wrap', position: 'relative', justifyContent: 'center', width: '-webkit-fill-available' }, ...other, children: [items, suitableCls > 0 && fakeSons > 0 && (new Array(fakeSons)).fill(null).map((each, clm) => {
                return _jsx("div", { css: { flex: `1 1 ${Math.round(10000 / suitableCls) / 100}%` } }, clm);
            })] }));
};
LineColumn.displayName = "LineColumn";
LineColumn.propTypes = {
    children: PropTypes.node
};
//# sourceMappingURL=LineColumn.js.map